/******************************************************************************
 * @file     main.c
 * @brief    Main program body
 * @version  V1.8.1
 * @date     9-Feb-2023
 *
 * @note
 * Copyright (C) 2022 Spintrol Electronic Technology (Shanghai) Co., Ltd.. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include<stdio.h>

#if defined(SPC1158)
#include "spc1158.h"
#elif defined(SPD1148)
#include "spd1148.h"
#elif defined(SPD1178)
#include "spd1178.h"
#elif defined(SPD1188)
#include "spd1188.h"
#elif defined(SPD1163)
#include "spd1163.h"
#else
#include "spc1168.h"
#endif

/*As descripted below, ADC result convert to voltage can calculate as the follow*/
#define             ValueToVoltage(x)         ((x * 3657) / 8192) /1000

#if defined(SPD1163)
#define             SOC_0_PinSel                ADC0_GPIO0
#define             SOC_1_PinSel                ADC4_GPIO4
#else
#define             SOC_0_PinSel                ADC10_GPIO10
#define             SOC_1_PinSel                ADC12_GPIO12
#endif

int16_t             i16VSP;


/*************************************************************************************************************************
*
* @brief      In this case, giving the GPIO10 or GPIO0/GPIO12 or GPIO4 a voltage and sample the voltage through Sampler A and B sequentially.
*
*                                  The KEY point :
*                                                     ADC result calculation formula:
*
*                                                                    (AIP-AIN)
*                                                     ADC_result = ----------- * 8192
*                                                                      3.657
*
*                                       GPIO10 or GPIO0       Sampler A
*                                                     SOC0
*                                        -            |\
*                                       | |           | \
*                                       | |-----------| /-----------RESULT[0]
*                                        -            |/
*
*                                                    Sampler B
*                                       GPIO12 or GPIO4         SOC1
*                                        -            |\
*                                       | |           | \
*                                       | |-----------| /-----------RESULT[1]
*                                        -            |/
*
*************************************************************************************************************************/


int main()
{
    FLASH_WALLOW();

#if defined(SPC1158)
    FLASH_SetTiming(100000000);
    /* Disable flash write access after flash operation had done */
    FLASH_WDIS();

    CLOCK_InitWithRCO(CLOCK_HCLK_100MHZ);
#else
    FLASH_SetTiming(200000000);
    /* Disable flash write access after flash operation had done */
    FLASH_WDIS();

    CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);
#endif

    Delay_Init();

    /*
     * Init the UART
     *
     * 1.Set the GPIO34/35 as UART FUNC
     *
     * 2.Enable the UART CLK
     *
     * 3.Set the rest para
     */
    GPIO_SetPinChannel(GPIO_34, GPIO34_UART_TXD);
    GPIO_SetPinChannel(GPIO_35, GPIO35_UART_RXD);
    CLOCK_EnableModule(UART_MODULE);
    UART_Init(UART, 38400);

    /* ADC Init, Set GPIO10 or GPIO0/GPIO12 or GPIO4 as the ADC SOC0/1 Positive pin respectively, the Negative pin of them is GND*/
    ADC_EasyInit1(ADC_SOC_0, SOC_0_PinSel, ADCTRIG_Software);
    ADC_EasyInit1(ADC_SOC_1, SOC_1_PinSel, ADCTRIG_Software);

    /* Enable G-INT for SOC0 and SOC1 */
    NVIC_EnableIRQ(ADC0_IRQn);
    NVIC_EnableIRQ(ADC1_IRQn);

    /* Use software to trigger ADC SOC to work */
    ADC_SoftwareTrigger(ADC_SOC_0);
    ADC_SoftwareTrigger(ADC_SOC_1);

    Delay_Ms(500);

    while (1)
    {

    }
}

void ADC0_IRQHandler(void)
{
    /* Result gotten from 'ADC_GetResult()' can be a negtive value or a positive value*/
    i16VSP = ADC_GetResult(ADC_SOC_0);

    printf("ADC SOC0 Voltage = %2fV(Raw data is %d)\n", (double)ValueToVoltage(i16VSP), i16VSP);

    /* Result gotten from 'ADC_GetTrimResult1()' is a positive value */
    i16VSP = ADC_GetTrimResult1(ADC_SOC_0);

    printf("ADC SOC0 Voltage = %2fV(Trim data is %d)\n", (double)ValueToVoltage(i16VSP), i16VSP);

    /* Clear SOC0 INT flag */
    ADC_ClearInt(ADC_SOC_0);
}

void ADC1_IRQHandler(void)
{
    /* Result gotten from 'ADC_GetResult()' can be a negtive value or a positive value */
    i16VSP = ADC_GetResult(ADC_SOC_1);

    printf("ADC SOC1 Voltage = %2fV(Raw data is %d)\n", (double)ValueToVoltage(i16VSP), i16VSP);

    /* Result gotten from 'ADC_GetTrimResult1()' is a positive value */
    i16VSP = ADC_GetTrimResult1(ADC_SOC_1);

    printf("ADC SOC1 Voltage = %2fV(Trim data is %d)\n", (double)ValueToVoltage(i16VSP), i16VSP);

    /* Clear SOC1 INT flag */
    ADC_ClearInt(ADC_SOC_1);
}

