/**************************************************************************//**
 * @file     main.c
 * @brief    Main program body
 * @version  V1.8.1
 * @date     9-Feb-2023
 *
 * @note
 * Copyright (C) 2022 Spintrol Electronic Technology (Shanghai) Co., Ltd.. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING 
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE 
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE 
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include "spd1163.h"
#include <stdio.h>

/* This macro is used to get the PWM period with a specified frequency */
#define                     PWMPeriod(u32PWMFreqHz)           ((CLOCK_GetModuleClock(PWM_MODULE))/u32PWMFreqHz)/2;

#define                     PRE_DRIVER_PWM_U                             PWM2
#define                     PRE_DRIVER_PWM_V                             PWM1
#define                     PRE_DRIVER_PWM_W                             PWM0
#define                     PRE_DRIVER_LINK_PWM_U                        SEL_PWM2
#define                     PRE_DRIVER_PWM_FREQ                          10000                  /* 10kHz PWM */
#define                     PWM_DB_NS                                    1000                  /* 1us */

uint32_t                    u32PWMPeriod;


/*************************************************************************************************************************
 *
 * @brief      This case show how to use PWM waveform to control PRE-DRIVER, there gives sketch map about the relationship
 *             between MCU and PRE-DRIVER.
 *
 *                                ----------
 *                               |          |               ---------
 *                               |          |   PWM0~2     |   PRE   |
 *                               |   MCU    |--------------| DRIVER  |
 *                               |          |              |         |
 *                               |          |              |         |
 *                                ----------                ---------
 *
 *************************************************************************************************************************/


int main(void)
{
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  printf("Enter the test\n");
  
  /* Init PRE-DRIVER module */
  HV_Init();

  /* 
   * Init PWM0~2
   *
   * Warnning : 
   *           when PWM is used for PRE-DRIVER, the dead-band of the pair complementary waveform of one PWM channel
   *           must not equal zero, bigger than 600 is recommanded.
   */
  PWM_ComplementaryPairChannelInit(PRE_DRIVER_PWM_U, PRE_DRIVER_PWM_FREQ, PWM_DB_NS);
  PWM_ComplementaryPairChannelInit(PRE_DRIVER_PWM_V, PRE_DRIVER_PWM_FREQ, PWM_DB_NS);
  PWM_ComplementaryPairChannelInit(PRE_DRIVER_PWM_W, PRE_DRIVER_PWM_FREQ, PWM_DB_NS);

  /* 
   * PWM_U set CMPA and loading the same value to PWM_V/PWM_W synchronously, LINK must be set earlier than CAMP value is set,
   * otherwaise, the LINK function will be invalid.
   */
  PWM_LinkCMPA(PRE_DRIVER_PWM_V, PRE_DRIVER_LINK_PWM_U);
  PWM_LinkCMPA(PRE_DRIVER_PWM_W, PRE_DRIVER_LINK_PWM_U);

  /* Set output 75% duty waveform */
  u32PWMPeriod = PWMPeriod(PRE_DRIVER_PWM_FREQ);
  PWM_SetCMPA(PRE_DRIVER_PWM_U, u32PWMPeriod / 4);
  
  /* Set PWM of PRE-DRIVER 1 start to count */
  PWM_RunCounter(PRE_DRIVER_PWM_U);
  PWM_RunCounter(PRE_DRIVER_PWM_V);
  PWM_RunCounter(PRE_DRIVER_PWM_W);

  while(1)
  {
  }
}

